'use strict';

var _lodash = require('lodash.assign');

var _lodash2 = _interopRequireDefault(_lodash);

var _immutablediff = require('immutablediff');

var _immutablediff2 = _interopRequireDefault(_immutablediff);

var _lodash3 = require('lodash.foreach');

var _lodash4 = _interopRequireDefault(_lodash3);

var _lodash5 = require('lodash.isarray');

var _lodash6 = _interopRequireDefault(_lodash5);

var _lodash7 = require('lodash.keys');

var _lodash8 = _interopRequireDefault(_lodash7);

var _lodash9 = require('lodash.wrap');

var _lodash10 = _interopRequireDefault(_lodash9);

var _lodash11 = require('lodash.defer');

var _lodash12 = _interopRequireDefault(_lodash11);

var _utils = require('./utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/* eslint-disable no-console */
var Base = function () {
  function Base() {
    _classCallCheck(this, Base);

    this.state = null;
    this.actions = {};
    this.logging = {};
    this.changeHandlers = [];
    if (this.initialize) this.initialize();
  }

  Base.prototype.getState = function getState() {
    return this.state;
  };

  Base.prototype.setState = function setState(state) {
    this.logStateUpdate(this.state, state);
    this.state = state;
    this.emitChange();
  };

  Base.prototype.updateState = function updateState(updater) {
    this.setState(updater(this.state));
  };

  Base.prototype.mutate = function mutate(keyPath, value) {
    var kp = (0, _lodash6.default)(keyPath) ? keyPath : keyPath.split('.');
    var val = typeof value === 'function' ? value(this.getState().getIn(kp)) : value;
    this.setState(this.getState().setIn(kp, val));
  };

  Base.prototype.get = function get(keyPath) {
    var kp = (0, _lodash6.default)(keyPath) ? keyPath : keyPath.split('.');
    return this.getState().getIn(kp);
  };

  Base.prototype.addActions = function addActions(name, ActionsClass) {
    this.actions[name] = new ActionsClass(this);
  };

  Base.prototype.getActions = function getActions(name) {
    if (name) return this.actions[name];
    return (0, _lodash2.default)({}, this.actions);
  };

  Base.prototype.enableLogging = function enableLogging(_ref) {
    var actions = _ref.actions;
    var state = _ref.state;
    var components = _ref.components;

    if (actions) {
      this.logging.actions = true;
      this.wrapActionMethods();
    }

    if (state) this.logging.state = true;
    if (components) this.logging.components = true;
  };

  Base.prototype.wrapActionMethods = function wrapActionMethods() {
    var logActionCall = this.logActionCall.bind(this);
    (0, _lodash4.default)(this.actions, function (actions, actionsName) {
      var methods = (0, _lodash8.default)(_utils2.default.getInternalMethods(actions.constructor));

      (0, _lodash4.default)(methods, function (method) {
        actions[method] = (0, _lodash10.default)(actions[method], function (func) {
          var callArgs = Array.prototype.slice.call(arguments, 1);
          logActionCall(actionsName, method, callArgs);
          return func.apply(this, callArgs);
        });
      });
    });
  };

  Base.prototype.logActionCall = function logActionCall(actionsName, methodName, args) {
    if (!this.logging.actions) return;

    var logArgs = ['%cAction performed', 'font-weight: bold;', actionsName + '.' + methodName].concat(args);
    console.log.apply(console, logArgs);
  };

  Base.prototype.logStateUpdate = function logStateUpdate(currentState, nextState) {
    if (!this.logging.state) return;

    if (console.group) console.group('%cState mutation', 'font-weight: bold;');

    console.log('%cCurrent state', 'color: gray; font-weight: bold; %O', currentState.toJS());
    var operations = (0, _immutablediff2.default)(currentState, nextState);
    if (operations.size > 1) {
      console.log('%cOperations %O', 'font-weight: bold;', (0, _immutablediff2.default)(currentState, nextState).toJS());
    } else if (operations.size === 1) {
      var operation = operations.first().toJS();
      console.log('%cOperation', 'font-weight: bold;', operation.op, operation.path, operation.value);
    } else {
      console.log('%cOperation', 'font-weight: bold;', 'Noop');
    }
    console.log('%cNext state', 'color: green; font-weight: bold; %O', nextState ? nextState.toJS() : '');

    if (console.groupEnd) console.groupEnd();
  };

  Base.prototype.logComponentRender = function logComponentRender(source, component) {
    if (!this.logging.components) return;
    console.log('%cComponent', 'font-weight: bold;', 'Re-render from ' + source, component);
  };

  Base.prototype.onChange = function onChange(handler) {
    this.changeHandlers.push(handler);
  };

  Base.prototype.offChange = function offChange(handler) {
    var _this = this;

    (0, _lodash4.default)(this.changeHandlers, function (fn, i) {
      if (handler === fn) {
        _this.changeHandlers.splice(i, 1);
        return false;
      }
    });
  };

  Base.prototype.emit = function emit() {
    (0, _lodash4.default)(this.changeHandlers, function (fn) {
      fn && fn();
    });
  };

  Base.prototype.emitChange = function emitChange() {
    var _this2 = this;

    if (this._willEmit) return;
    this._willEmit = true;
    (0, _lodash12.default)(function () {
      _this2._willEmit = false;
      _this2.emit();
    });
  };

  return Base;
}();
/* eslint-enable no-console */

module.exports = Base;