'use strict';

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _shallowEqual = require('react-pure-render/shallowEqual');

var _shallowEqual2 = _interopRequireDefault(_shallowEqual);

var _lodash = require('lodash.assign');

var _lodash2 = _interopRequireDefault(_lodash);

var _lodash3 = require('lodash.isarray');

var _lodash4 = _interopRequireDefault(_lodash3);

var _lodash5 = require('lodash.mapvalues');

var _lodash6 = _interopRequireDefault(_lodash5);

var _lodash7 = require('lodash.omit');

var _lodash8 = _interopRequireDefault(_lodash7);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defaults(obj, defaults) { var keys = Object.getOwnPropertyNames(defaults); for (var i = 0; i < keys.length; i++) { var key = keys[i]; var value = Object.getOwnPropertyDescriptor(defaults, key); if (value && value.configurable && obj[key] === undefined) { Object.defineProperty(obj, key, value); } } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : _defaults(subClass, superClass); }

var displayName = 'TideComponent';
var contextTypes = { tide: _react2.default.PropTypes.object };
var childContextTypes = { tide: _react2.default.PropTypes.object };
var propTypes = {
  impure: _react2.default.PropTypes.bool,
  tide: _react2.default.PropTypes.object
};

var NOT_KEY_PATH_PROPS = ['children', 'tide', 'impure', 'key', 'ref'];

var excludedProps = NOT_KEY_PATH_PROPS.reduce(function (val, prop) {
  val[prop] = true;
  return val;
}, {});

var Component = function (_React$Component) {
  _inherits(Component, _React$Component);

  function Component(props, context) {
    _classCallCheck(this, Component);

    var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

    var tide = props.tide || context.tide;
    _this._keyPaths = _this.getKeyPaths(props);
    _this._componentTide = {
      keyPaths: _this._keyPaths,
      actions: tide.getActions()
    };

    _this.state = _this.getPropsFromKeyPaths(_this._keyPaths, tide);
    return _this;
  }

  Component.prototype.getChildContext = function getChildContext() {
    return { tide: this.getTide() };
  };

  Component.prototype.componentWillMount = function componentWillMount() {
    this._onStateChange = this.onStateChange.bind(this);
    this.getTide().onChange(this._onStateChange);
  };

  Component.prototype.componentWillReceiveProps = function componentWillReceiveProps(newProps) {
    this._keyPaths = this.getKeyPaths(newProps);
  };

  Component.prototype.componentWillUnmount = function componentWillUnmount() {
    this._isUnmounting = true;
    this.getTide().offChange(this._onStateChange);
  };

  Component.prototype.onStateChange = function onStateChange() {
    if (this._isUnmounting) return;
    this._keyPaths = this.getKeyPaths(this.props);
    var newState = this.getPropsFromKeyPaths(this._keyPaths, this.getTide());
    if (this.hasStaleState(newState)) {
      this.setState(newState);
    }
  };

  Component.prototype.getKeyPaths = function getKeyPaths(props) {
    var keyPaths = (0, _lodash8.default)(props, function (value, key) {
      return excludedProps[key];
    });
    keyPaths = (0, _lodash6.default)(keyPaths, function (value, key) {
      if ((0, _lodash4.default)(value)) return value;
      if (value === true) return [key];
      return value.split('.');
    });
    return keyPaths;
  };

  Component.prototype.getPropsFromKeyPaths = function getPropsFromKeyPaths(keyPaths, tide) {
    var state = tide.getState();
    return (0, _lodash6.default)(keyPaths, function (value) {
      var last = value[value.length - 1];
      if (last === 'toJS()') {
        var obj = state.getIn(value.slice(0, -1));
        if (obj) return obj.toJS();
      }
      return state.getIn(value);
    });
  };

  Component.prototype.getTide = function getTide() {
    return this.props.tide || this.context.tide;
  };

  Component.prototype.getChildProps = function getChildProps() {
    return (0, _lodash2.default)((0, _lodash8.default)(this.state, function (value) {
      return value === undefined;
    }), { tide: this._componentTide });
  };

  Component.prototype.hasStaleState = function hasStaleState(newState) {
    return !(0, _shallowEqual2.default)(this.state, newState);
  };

  Component.prototype.wrapChild = function wrapChild(child) {
    return _react2.default.cloneElement(child, this.getChildProps());
  };

  Component.prototype.render = function render() {
    return _react2.default.Children.count(this.props.children) === 1 ? this.wrapChild(this.props.children) : _react2.default.createElement('span', null, _react2.default.Children.map(this.props.children, this.wrapChild.bind(this)));
  };

  return Component;
}(_react2.default.Component);

Component.displayName = displayName;
Component.propTypes = propTypes;
Component.contextTypes = contextTypes;
Component.childContextTypes = childContextTypes;

module.exports = Component;